"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .userschema import UserSchema, UserSchemaTypedDict
from owasp_nest.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class PagedUserSchemaTypedDict(TypedDict):
    items: List[UserSchemaTypedDict]
    count: int


class PagedUserSchema(BaseModel):
    items: List[UserSchema]

    count: int



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from owasp_nest.types import BaseModel
from typing_extensions import TypedDict


class LabelSchemaTypedDict(TypedDict):
    r"""Schema for Label."""

    color: str
    description: str
    name: str


class LabelSchema(BaseModel):
    r"""Schema for Label."""

    color: str

    description: str

    name: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from typing import TYPE_CHECKING
from importlib import import_module
import builtins

if TYPE_CHECKING:
    from .chapterschema import ChapterSchema, ChapterSchemaTypedDict
    from .committeeschema import CommitteeSchema, CommitteeSchemaTypedDict
    from .eventschema import EventSchema, EventSchemaTypedDict
    from .get_userop import GetUserRequest, GetUserRequestTypedDict
    from .issueschema import IssueSchema, IssueSchemaTypedDict
    from .labelschema import LabelSchema, LabelSchemaTypedDict
    from .list_chaptersop import (
        ListChaptersOrdering,
        ListChaptersRequest,
        ListChaptersRequestTypedDict,
    )
    from .list_committeesop import (
        ListCommitteesOrdering,
        ListCommitteesRequest,
        ListCommitteesRequestTypedDict,
    )
    from .list_eventsop import (
        ListEventsOrdering,
        ListEventsRequest,
        ListEventsRequestTypedDict,
    )
    from .list_issuesop import (
        ListIssuesOrdering,
        ListIssuesRequest,
        ListIssuesRequestTypedDict,
    )
    from .list_labelsop import (
        ListLabelsOrdering,
        ListLabelsRequest,
        ListLabelsRequestTypedDict,
    )
    from .list_organizationsop import (
        ListOrganizationsOrdering,
        ListOrganizationsRequest,
        ListOrganizationsRequestTypedDict,
    )
    from .list_projectsop import (
        ListProjectsOrdering,
        ListProjectsRequest,
        ListProjectsRequestTypedDict,
    )
    from .list_releasesop import (
        ListReleasesOrdering,
        ListReleasesRequest,
        ListReleasesRequestTypedDict,
    )
    from .list_repositoriesop import (
        ListRepositoriesOrdering,
        ListRepositoriesRequest,
        ListRepositoriesRequestTypedDict,
    )
    from .list_usersop import (
        ListUsersOrdering,
        ListUsersRequest,
        ListUsersRequestTypedDict,
    )
    from .nestapierror import NestAPIError
    from .nesterror import NestError
    from .no_response_error import NoResponseError
    from .organizationschema import OrganizationSchema, OrganizationSchemaTypedDict
    from .pagedchapterschema import PagedChapterSchema, PagedChapterSchemaTypedDict
    from .pagedcommitteeschema import (
        PagedCommitteeSchema,
        PagedCommitteeSchemaTypedDict,
    )
    from .pagedeventschema import PagedEventSchema, PagedEventSchemaTypedDict
    from .pagedissueschema import PagedIssueSchema, PagedIssueSchemaTypedDict
    from .pagedlabelschema import PagedLabelSchema, PagedLabelSchemaTypedDict
    from .pagedorganizationschema import (
        PagedOrganizationSchema,
        PagedOrganizationSchemaTypedDict,
    )
    from .pagedprojectschema import PagedProjectSchema, PagedProjectSchemaTypedDict
    from .pagedreleaseschema import PagedReleaseSchema, PagedReleaseSchemaTypedDict
    from .pagedrepositoryschema import (
        PagedRepositorySchema,
        PagedRepositorySchemaTypedDict,
    )
    from .pageduserschema import PagedUserSchema, PagedUserSchemaTypedDict
    from .projectlevel import ProjectLevel
    from .projectschema import ProjectSchema, ProjectSchemaTypedDict
    from .releaseschema import ReleaseSchema, ReleaseSchemaTypedDict
    from .repositoryschema import RepositorySchema, RepositorySchemaTypedDict
    from .responsevalidationerror import ResponseValidationError
    from .security import Security, SecurityTypedDict
    from .state import State
    from .usererrorresponse import UserErrorResponse, UserErrorResponseData
    from .userschema import UserSchema, UserSchemaTypedDict

__all__ = [
    "ChapterSchema",
    "ChapterSchemaTypedDict",
    "CommitteeSchema",
    "CommitteeSchemaTypedDict",
    "EventSchema",
    "EventSchemaTypedDict",
    "GetUserRequest",
    "GetUserRequestTypedDict",
    "IssueSchema",
    "IssueSchemaTypedDict",
    "LabelSchema",
    "LabelSchemaTypedDict",
    "ListChaptersOrdering",
    "ListChaptersRequest",
    "ListChaptersRequestTypedDict",
    "ListCommitteesOrdering",
    "ListCommitteesRequest",
    "ListCommitteesRequestTypedDict",
    "ListEventsOrdering",
    "ListEventsRequest",
    "ListEventsRequestTypedDict",
    "ListIssuesOrdering",
    "ListIssuesRequest",
    "ListIssuesRequestTypedDict",
    "ListLabelsOrdering",
    "ListLabelsRequest",
    "ListLabelsRequestTypedDict",
    "ListOrganizationsOrdering",
    "ListOrganizationsRequest",
    "ListOrganizationsRequestTypedDict",
    "ListProjectsOrdering",
    "ListProjectsRequest",
    "ListProjectsRequestTypedDict",
    "ListReleasesOrdering",
    "ListReleasesRequest",
    "ListReleasesRequestTypedDict",
    "ListRepositoriesOrdering",
    "ListRepositoriesRequest",
    "ListRepositoriesRequestTypedDict",
    "ListUsersOrdering",
    "ListUsersRequest",
    "ListUsersRequestTypedDict",
    "NestAPIError",
    "NestError",
    "NoResponseError",
    "OrganizationSchema",
    "OrganizationSchemaTypedDict",
    "PagedChapterSchema",
    "PagedChapterSchemaTypedDict",
    "PagedCommitteeSchema",
    "PagedCommitteeSchemaTypedDict",
    "PagedEventSchema",
    "PagedEventSchemaTypedDict",
    "PagedIssueSchema",
    "PagedIssueSchemaTypedDict",
    "PagedLabelSchema",
    "PagedLabelSchemaTypedDict",
    "PagedOrganizationSchema",
    "PagedOrganizationSchemaTypedDict",
    "PagedProjectSchema",
    "PagedProjectSchemaTypedDict",
    "PagedReleaseSchema",
    "PagedReleaseSchemaTypedDict",
    "PagedRepositorySchema",
    "PagedRepositorySchemaTypedDict",
    "PagedUserSchema",
    "PagedUserSchemaTypedDict",
    "ProjectLevel",
    "ProjectSchema",
    "ProjectSchemaTypedDict",
    "ReleaseSchema",
    "ReleaseSchemaTypedDict",
    "RepositorySchema",
    "RepositorySchemaTypedDict",
    "ResponseValidationError",
    "Security",
    "SecurityTypedDict",
    "State",
    "UserErrorResponse",
    "UserErrorResponseData",
    "UserSchema",
    "UserSchemaTypedDict",
]

_dynamic_imports: dict[str, str] = {
    "ChapterSchema": ".chapterschema",
    "ChapterSchemaTypedDict": ".chapterschema",
    "CommitteeSchema": ".committeeschema",
    "CommitteeSchemaTypedDict": ".committeeschema",
    "EventSchema": ".eventschema",
    "EventSchemaTypedDict": ".eventschema",
    "GetUserRequest": ".get_userop",
    "GetUserRequestTypedDict": ".get_userop",
    "IssueSchema": ".issueschema",
    "IssueSchemaTypedDict": ".issueschema",
    "LabelSchema": ".labelschema",
    "LabelSchemaTypedDict": ".labelschema",
    "ListChaptersOrdering": ".list_chaptersop",
    "ListChaptersRequest": ".list_chaptersop",
    "ListChaptersRequestTypedDict": ".list_chaptersop",
    "ListCommitteesOrdering": ".list_committeesop",
    "ListCommitteesRequest": ".list_committeesop",
    "ListCommitteesRequestTypedDict": ".list_committeesop",
    "ListEventsOrdering": ".list_eventsop",
    "ListEventsRequest": ".list_eventsop",
    "ListEventsRequestTypedDict": ".list_eventsop",
    "ListIssuesOrdering": ".list_issuesop",
    "ListIssuesRequest": ".list_issuesop",
    "ListIssuesRequestTypedDict": ".list_issuesop",
    "ListLabelsOrdering": ".list_labelsop",
    "ListLabelsRequest": ".list_labelsop",
    "ListLabelsRequestTypedDict": ".list_labelsop",
    "ListOrganizationsOrdering": ".list_organizationsop",
    "ListOrganizationsRequest": ".list_organizationsop",
    "ListOrganizationsRequestTypedDict": ".list_organizationsop",
    "ListProjectsOrdering": ".list_projectsop",
    "ListProjectsRequest": ".list_projectsop",
    "ListProjectsRequestTypedDict": ".list_projectsop",
    "ListReleasesOrdering": ".list_releasesop",
    "ListReleasesRequest": ".list_releasesop",
    "ListReleasesRequestTypedDict": ".list_releasesop",
    "ListRepositoriesOrdering": ".list_repositoriesop",
    "ListRepositoriesRequest": ".list_repositoriesop",
    "ListRepositoriesRequestTypedDict": ".list_repositoriesop",
    "ListUsersOrdering": ".list_usersop",
    "ListUsersRequest": ".list_usersop",
    "ListUsersRequestTypedDict": ".list_usersop",
    "NestAPIError": ".nestapierror",
    "NestError": ".nesterror",
    "NoResponseError": ".no_response_error",
    "OrganizationSchema": ".organizationschema",
    "OrganizationSchemaTypedDict": ".organizationschema",
    "PagedChapterSchema": ".pagedchapterschema",
    "PagedChapterSchemaTypedDict": ".pagedchapterschema",
    "PagedCommitteeSchema": ".pagedcommitteeschema",
    "PagedCommitteeSchemaTypedDict": ".pagedcommitteeschema",
    "PagedEventSchema": ".pagedeventschema",
    "PagedEventSchemaTypedDict": ".pagedeventschema",
    "PagedIssueSchema": ".pagedissueschema",
    "PagedIssueSchemaTypedDict": ".pagedissueschema",
    "PagedLabelSchema": ".pagedlabelschema",
    "PagedLabelSchemaTypedDict": ".pagedlabelschema",
    "PagedOrganizationSchema": ".pagedorganizationschema",
    "PagedOrganizationSchemaTypedDict": ".pagedorganizationschema",
    "PagedProjectSchema": ".pagedprojectschema",
    "PagedProjectSchemaTypedDict": ".pagedprojectschema",
    "PagedReleaseSchema": ".pagedreleaseschema",
    "PagedReleaseSchemaTypedDict": ".pagedreleaseschema",
    "PagedRepositorySchema": ".pagedrepositoryschema",
    "PagedRepositorySchemaTypedDict": ".pagedrepositoryschema",
    "PagedUserSchema": ".pageduserschema",
    "PagedUserSchemaTypedDict": ".pageduserschema",
    "ProjectLevel": ".projectlevel",
    "ProjectSchema": ".projectschema",
    "ProjectSchemaTypedDict": ".projectschema",
    "ReleaseSchema": ".releaseschema",
    "ReleaseSchemaTypedDict": ".releaseschema",
    "RepositorySchema": ".repositoryschema",
    "RepositorySchemaTypedDict": ".repositoryschema",
    "ResponseValidationError": ".responsevalidationerror",
    "Security": ".security",
    "SecurityTypedDict": ".security",
    "State": ".state",
    "UserErrorResponse": ".usererrorresponse",
    "UserErrorResponseData": ".usererrorresponse",
    "UserSchema": ".userschema",
    "UserSchemaTypedDict": ".userschema",
}


def __getattr__(attr_name: str) -> object:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(
            f"No {attr_name} found in _dynamic_imports for module name -> {__name__} "
        )

    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(
            f"Failed to import {attr_name} from {module_name}: {e}"
        ) from e
    except AttributeError as e:
        raise AttributeError(
            f"Failed to get {attr_name} from {module_name}: {e}"
        ) from e


def __dir__():
    lazy_attrs = builtins.list(_dynamic_imports.keys())
    return builtins.sorted(lazy_attrs)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
import httpx
import importlib
from owasp_nest import models, utils
from owasp_nest._hooks import HookContext, SDKHooks
from owasp_nest.types import OptionalNullable, UNSET
from typing import Any, Callable, Dict, Mapping, Optional, TYPE_CHECKING, Union, cast
import weakref

if TYPE_CHECKING:
    from owasp_nest.github import GitHub
    from owasp_nest.owasp import Owasp


class Nest(BaseSDK):
    r"""OWASP Nest: Open Worldwide Application Security Project API"""

    owasp: "Owasp"
    git_hub: "GitHub"
    _sub_sdk_map = {
        "owasp": ("owasp_nest.owasp", "Owasp"),
        "git_hub": ("owasp_nest.github", "GitHub"),
    }

    def __init__(
        self,
        api_key_auth: Union[str, Callable[[], str]],
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param api_key_auth: The api_key_auth required for authentication
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client()
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient()
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(api_key_auth):
            # pylint: disable=unnecessary-lambda-assignment
            security = lambda: models.Security(api_key_auth=api_key_auth())
        else:
            security = models.Security(api_key_auth=api_key_auth)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                security=security,
                server_url=server_url,
                server_idx=server_idx,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
        )

        hooks = SDKHooks()

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        self.sdk_configuration = hooks.sdk_init(self.sdk_configuration)

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

    def __getattr__(self, name: str):
        if name in self._sub_sdk_map:
            module_path, class_name = self._sub_sdk_map[name]
            try:
                module = importlib.import_module(module_path)
                klass = getattr(module, class_name)
                instance = klass(self.sdk_configuration)
                setattr(self, name, instance)
                return instance
            except ImportError as e:
                raise AttributeError(
                    f"Failed to import module {module_path} for attribute {name}: {e}"
                ) from e
            except AttributeError as e:
                raise AttributeError(
                    f"Failed to find class {class_name} in module {module_path} for attribute {name}: {e}"
                ) from e

        raise AttributeError(
            f"'{type(self).__name__}' object has no attribute '{name}'"
        )

    def __dir__(self):
        default_attrs = list(super().__dir__())
        lazy_attrs = list(self._sub_sdk_map.keys())
        return sorted(list(set(default_attrs + lazy_attrs)))

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None

    def settings_api_v1_api_root(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Api Root

        Handle API root endpoint requests.

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request(
            method="GET",
            path="/api/v1/",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="settings_api_v1_api_root",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.NestError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.NestError("API error occurred", http_res, http_res_text)

        raise models.NestError("Unexpected response received", http_res)

    async def settings_api_v1_api_root_async(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Api Root

        Handle API root endpoint requests.

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request_async(
            method="GET",
            path="/api/v1/",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="settings_api_v1_api_root",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.NestError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.NestError("API error occurred", http_res, http_res_text)

        raise models.NestError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .labelschema import LabelSchema, LabelSchemaTypedDict
from owasp_nest.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class PagedLabelSchemaTypedDict(TypedDict):
    items: List[LabelSchemaTypedDict]
    count: int


class PagedLabelSchema(BaseModel):
    items: List[LabelSchema]

    count: int
